import { Injectable } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Observable } from "rxjs";
import { jwtDecode } from "jwt-decode";
import { Router } from "@angular/router";
import { AuthService } from "src/app/_services/auth.service";

const BASE_URL = "https://kapi.absys.ninja/hemat";
interface CustomJwtPayload {
  exp: number;
  scope: string;
  iat: number;
  preferred_username: string;
  name: string;
  email: string;
  family_name: string;
  given_name: string;
  sub: string;
}

@Injectable({
  providedIn: "root",
})
export class LoginService {
  private readonly tokenKey = "currentUser";
  constructor(
    private http: HttpClient,
    private router: Router,
    public logoutService: AuthService
  ) {}

  updatePassword(data: any): Observable<any> {
    const endpoint = `/users`;
    const url = `${BASE_URL}${endpoint}/reset-password`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.put<any>(url, data, { headers });
  }

  getDataProfil(id): Observable<any> {
    const endpoint = `/users`;
    const url = `${BASE_URL}${endpoint}/byUserid/${id}`;
    const headers = new HttpHeaders({
      "Content-Type": "application/json",
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });
    return this.http.get<any>(url, { headers });
  }

  updateProfile(data: any, userId: string): Observable<any> {
    const endpoint = `/users`;
    const url = `${BASE_URL}${endpoint}/update/${userId}`;
    const headers = new HttpHeaders({
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });

    return this.http.post(`${url}`, data, { headers });
  }

  updateUserProfile(data: any): Observable<any> {
    const body = { refresh_token: data };
    const endpoint = `/users`;
    const url = `${BASE_URL}${endpoint}/refresh-token`;
    const headers = new HttpHeaders({
      "x-api-key": "j2yaYvPSQcsEEmHh3NEobfiXyyXmmnHT",
    });

    return this.http.post(`${url}`, body, { headers });
  }

  isTokenExpired(): boolean {
    const tokenData = localStorage.getItem(this.tokenKey);
    if (tokenData) {
      const tokenInfo = JSON.parse(tokenData);
      const decodedToken = jwtDecode<CustomJwtPayload>(tokenInfo.refresh_token);
      const expiryDate = decodedToken.exp * 1000;
      const now = new Date().getTime();
      return now > expiryDate;
    }
    return true;
  }

  checkTokenAndRedirect(): void {
    if (this.isTokenExpired()) {
      // Token sudah kedaluwarsa, arahkan ke halaman login
      // console.log("Token expired, redirecting to login page...");
      this.logoutService.doLogout().then(
        (res) => {
          this.router.navigate(["/login"]);
        },
        (err) => {
          console.log(err);
        }
      );
      this.router.navigate(["/login"]);
    } else {
      console.log("Token is valid, continuing...");
    }
  }
}
